#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <time.h>

#include "amcc.h"

/* ---------------------------------- */
/*      DSP register offsets          */
/* ---------------------------------- */

#define ISASEL_MEM     ((unsigned*)0xd8000000L)      /* 2 bytes */
#define ISASEL_PAGEREG ((unsigned*)0xda000000L)
#define ISASEL_CPLD    ((unsigned*)0xda000000L)
#define ISASEL_PIO     ((unsigned*)0xd9000000L)
#define ISASEL_AON     ((unsigned*)0xdc000000L)
#define ISASEL_TMR     ((char*)0xdb000000L)          /* 1 byte */

/* --- keyboard interrupt routine --- */
#ifdef __cplusplus
    #define __CPPARGS ...
#else
    #define __CPPARGS
#endif

void interrupt get_out(__CPPARGS); /*    interrupt prototype */
void interrupt (*oldfunc)(__CPPARGS); /* interrupt function pointer */

int  isamem_low_walking_bit_test( );
int  isamem_high_walking_bit_test( );
int  isamem_write_validate();
int  isamem_full_test();
int  isamem_endless_loop();
void isamem_short_test();

int looping;



/* --------------------------------- */
/*   isamem_low_walking_bit_test( )  */
/*   ISA Memory Lower 16 bits   */
/*   Walking 1's and 0's        */
/* ---------------------------- */

int isamem_low_walking_bit_test( )
{
 unsigned read_u_val;       /* 2 bytes */
 unsigned write_u_val;
 int fail;
 int bit_place;
 char key;

 printf ("\n\n");
 printf ("ISA Memory Lower 16 bits, Walking 1's and 0's\n");

 write_u_val = 1;
 fail = 0;
 *ISASEL_PAGEREG = 0;

 /* walking 1's and 0's*/

 printf ("   Write    Read       Write    Read\n");
 printf ("   Value   Value       Value   Value\n\n");

 for ( bit_place = 0; bit_place < 16; bit_place++ ) {
     *ISASEL_MEM = write_u_val;
     read_u_val = *ISASEL_MEM;
     if (  (unsigned) (0xffff & read_u_val) !=
	   (unsigned) (0xffff & write_u_val) )
     fail = fail + 1;

     printf ("%8x%8x    ", 0xffff & write_u_val, 0xffff & read_u_val );

     *ISASEL_MEM = ~write_u_val;
     read_u_val = *ISASEL_MEM;
     if ( (unsigned) (0xffff & read_u_val) !=
	   (unsigned) (0xffff & ~write_u_val) )
	  fail = fail + 1;

     printf ("%8x%8x\n", 0xffff & ~write_u_val, 0xffff & read_u_val);
     write_u_val = write_u_val * 2;
     }

 if ( fail )
   printf ("test fails, number of failures = %d\n", fail);
 else
   printf ("test passes\n");

 printf ("type ENTER key to continue\n");
 scanf ( "%c", &key );

 }

/* ---------------------------------- */
/*   isamem_high_walking_bit_test( )  */
/*   Memory Upper 16 bits             */
/*   Walking 1's and 0's              */
/* ---------------------------------- */

int isamem_high_walking_bit_test( )
{
 unsigned read_u_val;       /* 2 bytes */
 unsigned write_u_val;
 int fail;
 int bit_place;
 char key;

 printf ("\n\n");
 printf ("ISA Memory Upper 16 bits, Walking 1's and 0's\n");

 write_u_val = 1;
 fail = 0;
 *ISASEL_PAGEREG = 0;

 /* walking 1's and 0's*/

 printf ("   Write    Read       Write    Read\n");
 printf ("   Value   Value       Value   Value\n\n");

 for ( bit_place = 0; bit_place < 16; bit_place++ ) {
     *(ISASEL_MEM+1) = write_u_val;
     read_u_val = *(ISASEL_MEM+1);
     if (  (unsigned) (0xffff & read_u_val) !=
	   (unsigned) (0xffff & write_u_val) )
        fail = fail + 1;

     printf ("%8x%8x    ", 0xffff & write_u_val, 0xffff & read_u_val );

     *(ISASEL_MEM+1) = ~write_u_val;
     read_u_val = *(ISASEL_MEM+1);
     if ( (unsigned) (0xffff & read_u_val) !=
	   (unsigned) (0xffff & ~write_u_val) )
	 fail = fail + 1;

     printf ("%8x%8x\n", 0xffff & ~write_u_val, 0xffff & read_u_val);
     write_u_val = write_u_val * 2;
     }

 if ( fail )
   printf ("test fails, number of failures = %d\n", fail);
 else
   printf ("test passes\n");

 printf ("type ENTER key to continue\n");
 scanf ( "%c", &key );
}

/* ---------------------------- */
/*     isamem_write_validate()  */
/*     ISA MEMORY TEST          */
/*     Write and Validate       */
/* ---------------------------- */

int isamem_write_validate()
{
 unsigned read_u_val;       /* 2 bytes */
 unsigned write_u_val;
 int fail;
 int i;

 printf ("TEST NUMBER, ISA WRITE/VALIDATE Memory Test\n");
   *ISASEL_PAGEREG = 0;
   srand(1);
   fail = 0;

   for ( i = 0; i < 10 ; i++ ){
      write_u_val  = (unsigned long)(0xff & rand());
      write_u_val += (unsigned long)(0xff & rand()) << 8;
      *(ISASEL_MEM+i) = write_u_val;
   }

  srand(1);
  printf ("   Write    Read\n");
  printf ("   Value   Value\n");

   for ( i = 0; i < 10 ; i++ ){
      write_u_val  = (unsigned long)(0xff & rand());
      write_u_val += (unsigned long)(0xff & rand()) << 8;
      read_u_val = *(ISASEL_MEM+i);
      if ( (unsigned) read_u_val != (unsigned) write_u_val )
		fail = fail + 1;
      printf("%8x  %8x\n", write_u_val, read_u_val );
      }

 if ( fail )
   printf ("test fails, number of failures = %d\n", fail);
 else
   printf ("test passes\n");
}


/* --------------------------- */
/*  isamem_full_test()         */
/*  ISA FULL Memory Test       */
/* --------------------------- */
void ctrl_fill_memory(int seed)
{
 unsigned read_u_val;       /* 2 bytes */
 unsigned write_u_val;
 int i, j;

 printf ("TEST NUMBER, ISA Full Memory Test\n");

/* write a random pattern */

srand(seed);

for ( j = 0; j < 256 ; j++ ){
   *ISASEL_PAGEREG = j;
   for ( i = 0; i < 2048 ; i++ ){
      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      *(ISASEL_MEM+i) = write_u_val;
      }
   }

}

int ctrl_validate_memory(int seed)
{
 unsigned read_u_val;       /* 2 bytes */
 unsigned write_u_val;
 int fail;
 int i, j;

/* read back the pattern and compare */
srand(seed);
fail = 0;

for ( j = 0; j < 256 ; j++ ){
   *ISASEL_PAGEREG = j;
   for ( i = 0; i < 2048 ; i++ ){
      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      read_u_val = *(ISASEL_MEM+i);

      if (  read_u_val !=  write_u_val )
	       fail = fail + 1;
   }
}

return(fail);
}


void ctrl_fill_memory_mem(int seed)
{
 unsigned read_u_val;       /* 2 bytes */
 unsigned write_u_val;
 int i, j;

 printf ("TEST NUMBER, ISA Full Memory Test\n");

/* write a random pattern */

srand(seed);

for ( j = 1; j < 256 ; j++ ){
   *ISASEL_PAGEREG = j;
   for ( i = 0; i < 2048 ; i++ ){
      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      *(ISASEL_MEM+i) = write_u_val;
      }
   }

}

int ctrl_validate_memory_mem(int seed)
{
 unsigned read_u_val;       /* 2 bytes */
 unsigned write_u_val;
 int fail;
 int i, j;

/* read back the pattern and compare */
srand(seed);
fail = 0;

for ( j = 1; j < 256 ; j++ ){
   *ISASEL_PAGEREG = j;
   for ( i = 0; i < 2048 ; i++ ){
      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      read_u_val = *(ISASEL_MEM+i);

      if (  read_u_val !=  write_u_val )
	       fail = fail + 1;
   }
}

return(fail);
}


/* --------------------------- */
/*  isamem_endless_loop ()     */
/*  ISA FULL Memory Test       */
/*  Endless Loop               */
/* --------------------------- */

int isamem_endless_loop ()
{
 unsigned read_u_val;       /* 2 bytes */
 unsigned write_u_val;
 int fail;
 int i, j;


 printf ("TEST NUMBER, ISA Full Memory Test, Endless Loop\n");

 puts("Press <Shift><PrtSc> to terminate");

 /* save the old interrupt */
 oldfunc  = _dos_getvect(5);

 /* install interrupt handler */
 _dos_setvect(5,get_out);

 looping = 1;

 while ( looping ) {

 fail = 0;

 /* write a random pattern */

 srand(1);

 for ( j = 0; j < 256 ; j++ ){
   *ISASEL_PAGEREG = j;
   for ( i = 0; i < 2048 ; i++ ){
      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      *(ISASEL_MEM+i) = write_u_val;
      }
   }

 /* read back the pattern and compare */
 srand(1);

 for ( j = 0; j < 256 ; j++ ){
   *ISASEL_PAGEREG = j;
   for ( i = 0; i < 2048 ; i++ ){
      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      read_u_val = *(ISASEL_MEM+i);

      if (  read_u_val !=  write_u_val ){
          *(ISASEL_TMR+3) = 0x76;   /* command word, counter 1*/
	       fail = fail + 1;
          }
   }
}

if ( fail )
  printf ("test fails, number of failures = %d\n", fail);
else
  printf ("test passes\n");


} /* while ( looping ) */

  /* restore to original interrupt routine */
  _dos_setvect(5,oldfunc);

  puts("Success");

}

void isamem_short_test()
{
 unsigned read_u_val;       /* 2 bytes */
 unsigned write_u_val;
 int fail;
 char key;
 int i;
 
 printf ("Memory(ISA) Short Test\n");

 *ISASEL_PAGEREG = 0;
 srand(1);
 fail = 0;

   for ( i = 0; i < 10 ; i++ ){
      write_u_val  = (unsigned long)(0xff & rand());
      write_u_val += (unsigned long)(0xff & rand()) << 8;
      *(ISASEL_MEM+i) = write_u_val;
   }

  srand(1);
  printf ("   Write    Read\n");
  printf ("   Value   Value\n");

   for ( i = 0; i < 10 ; i++ ){
      write_u_val  = (unsigned long)(0xff & rand());
      write_u_val += (unsigned long)(0xff & rand()) << 8;
      read_u_val = *(ISASEL_MEM+i);
      if ( (unsigned) read_u_val != (unsigned) write_u_val )
		fail = fail + 1;
      printf("%8x  %8x\n", write_u_val, read_u_val );
      }

     if ( fail )
       printf ("test fails, number of failures = %d\n", fail);
     else
       printf ("test passes\n");

   printf ("type ENTER key to continue\n");
   scanf ( "%c", &key );

}

void interrupt get_out(__CPPARGS) {
  looping = 0;             /* change global var to get out of oop */
}
