#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <time.h>

#include "amcc.h"

/* ---------------------------------- */
/*      DSP register offsets          */
/* ---------------------------------- */

#define ISASEL_MEM     ((unsigned*)0xd8000000L)      /* 2 bytes */
#define ISASEL_PAGEREG ((unsigned*)0xda000000L)
#define ISASEL_CPLD    ((unsigned*)0xda000000L)
#define ISASEL_PIO     ((unsigned*)0xd9000000L)
#define ISASEL_AON     ((unsigned*)0xdc000000L)
#define ISASEL_TMR     ((char*)0xdb000000L)          /* 1 byte */

/* Add-On Operation Registers */
#define AIMB1LOW  0x00   /* Add-On Incoming Mailbox Register #1 */
#define AIMB1HIGH 0x01   /* Add-On Incoming Mailbox Register #1 */
#define AIMB2LOW  0x02   /* Add-On Incoming Mailbox Register #2 */
#define AIMB2HIGH 0x03   /* Add-On Incoming Mailbox Register #2 */
#define AIMB3LOW  0x04   /* Add-On Incoming Mailbox Register #3 */
#define AIMB3HIGH 0x05   /* Add-On Incoming Mailbox Register #3 */
#define AIMB4LOW  0x06   /* Add-On Incoming Mailbox Register #4 */
#define AIMB4HIGH 0x07   /* Add-On Incoming Mailbox Register #4 */
#define AOMB1LOW  0x08   /* Add-On Outgoing Mailbox Register #1 */
#define AOMB1HIGH 0x09   /* Add-On Outgoing Mailbox Register #1 */
#define AOMB2LOW  0x0a   /* Add-On Outgoing Mailbox Register #2 */
#define AOMB2HIGH 0x0b   /* Add-On Outgoing Mailbox Register #2 */
#define AOMB3LOW  0x0c   /* Add-On Outgoing Mailbox Register #3 */
#define AOMB3HIGH 0x0d   /* Add-On Outgoing Mailbox Register #3 */
#define AOMB4LOW  0x0e   /* Add-On Outgoing Mailbox Register #4 */
#define AOMB4HIGH 0x0f   /* Add-On Outgoing Mailbox Register #4 */

#define AMBEFLOW	0x1A   /* Add-On Mailbox  Empty/Full Status */
#define AMBEFHIGH	0x1B   /* Add-On Mailbox  Empty/Full Status */


void init_timer();
void read_pio();
void write_pio();
void ctrl_ramp_test();

/* -------------------- */
/*   void init_timer()  */
/*   Init Timer         */
/*   Square Wave        */
/*   Period = 15.2uS    */
/* -------------------- */

/*
  Control Word Format:

  | D7| D6| D5| D4| D3| D2| D1| D0|
  |SC1|SC0|RW1|RW0| M2| M1| M0|BCD|
  |   |   |   |   |   |   |   |   |
  | 0 | 0 | 1 | 1 | 0 | 1 | 0 | 0 |   0x34 cntr0, LSbyte then MSbyte
  |   |   |   |   |   |   |   |   |        mode2, binary cntr
  | 0 | 1 | 1 | 1 | 0 | 1 | 0 | 0 |   0x74 cntr1, LSbyte then MSbyte
  |   |   |   |   |   |   |   |   |        mode2, binary cntr
  | 1 | 0 | 1 | 1 | 0 | 1 | 0 | 0 |   0xb4 cntr2, LSbyte then MSbyte
  |   |   |   |   |   |   |   |   |        mode2, binary cntr
  | 0 | 0 | 1 | 1 | 0 | 1 | 1 | 0 |   0x36 cntr0, LSbyte then MSbyte
  |   |   |   |   |   |   |   |   |        mode3, binary cntr
  | 0 | 1 | 1 | 1 | 0 | 1 | 1 | 0 |   0x76 cntr1, LSbyte then MSbyte
  |   |   |   |   |   |   |   |   |        mode3, binary cntr
  | 1 | 0 | 1 | 1 | 0 | 1 | 1 | 0 |   0xb6 cntr2, LSbyte then MSbyte
  |   |   |   |   |   |   |   |   |        mode3, binary cntr

  Read Back Command
  | D7| D6|  D5  |   D4  | D3 | D2 | D1 | D0|
  | 1 | 1 |COUNT*|STATUS*|CNT2|CNT1|CNT0| 0 |
  |   |   |      |       |    |    |    |   |
  | 1 | 1 |   0  |   0   |  1 |  1 |  1 | 0 |  0xce Latch Count and
  |   |   |      |       |    |    |    |   |       Status for cnt0
  |   |   |      |       |    |    |    |   |       cnt1, and cnt2
  | 1 | 1 |   0  |   1   |  0 |  0 |  1 | 0 |  0xd2 Latch cnt0
  |   |   |      |       |    |    |    |   |
  | 1 | 1 |   0  |   1   |  0 |  1 |  0 | 0 |  0xd4 Latch cnt1
  |   |   |      |       |    |    |    |   |
  | 1 | 1 |   0  |   1   |  1 |  0 |  0 | 0 |  0xd8 Latch cnt2
  |   |   |      |       |    |    |    |   |

*/

void init_timer()
{
 printf("init_timer\n");
 /* Write  Control Words to set counter mode */
 *(ISASEL_TMR+3) = 0x36;   /* counter 0 */
 *(ISASEL_TMR+3) = 0x76;   /* counter 1 */
 *(ISASEL_TMR+3) = 0xb6;   /* counter 2 */

 /* Write the initial count value, LSbyte then MSbyte */
 *(ISASEL_TMR+2) = 0x80;   /* counter 2 */
 *(ISASEL_TMR+2) = 0x00;
 *(ISASEL_TMR+1) = 0x80;   /* counter 1 */
 *(ISASEL_TMR+1) = 0x00;
 *(ISASEL_TMR)   = 0x80;   /* counter 0 */
 *(ISASEL_TMR)   = 0x00;
}


void read_pio()
{
 FILE *stream;
 int i, j;
 unsigned val[10000];
 char string[25];
 char key;

 init_timer();
 j = 0;

 while(1){
 printf ("type ENTER key to continue\n");
 scanf ( "%c", &key );


 for ( i=0; i < 256; i++) {
    while ( (*ISASEL_CPLD & 0x0001) == 0x00001 ) ;
       // loop till TMROUT0 = 0

    while ( (*ISASEL_CPLD & 0x0001) == 0x00000 ) ;
       // loop till TMROUT0 = 1

    val[i] = *(ISASEL_PIO + 0xc);
 }

 j++;
 itoa(j, string, 10);

 /* open a file for update */
 stream = fopen(string, "w+");

 for ( i=0; i < 256; i++ ){
 fprintf(stream, "%7d\n", val[i]);
 }

 /* close the file */
 fclose(stream);
 } // end while

}

/* ---------------------------- */
/* void write_pio()             */
/* generate sawtooth            */
/* positive ramp                */
/* period = 3.9 ms              */
/* peak to peak = +/- 10 Volts  */
/* ---------------------------- */

void write_pio()
{
 int i;
 char key;
 init_timer();

 while(1){
// printf ("type ENTER key to continue\n");
// scanf ( "%c", &key );

 for ( i=0; i < 256; i++) {
    while ( (*ISASEL_CPLD & 0x0001) == 0x00001 ) ;
       // loop till TMROUT0 = 0

    while ( (*ISASEL_CPLD & 0x0001) == 0x00000 ) ;
       // loop till TMROUT0 = 1

    *(ISASEL_PIO + 0x6) = (unsigned) i * 256;
    *(ISASEL_PIO + 0x7) = 0;
 }


 } // end while

}


/* --------------------------------------------- */
/* void ctrl_ramp_test()                         */
/* write putstream saw tooth to DAC              */
/* read back getstream from 16-bit A/D           */
/*      DAC = address 6                          */
/*   STROBE = address 7                          */
/*      A/D = address 2                          */
/*                                               */
/* PAGE 1    put stream saw tooth                */
/* PAGE 2    put stream saw tooth (copy)         */
/* PAGE 3    get stream                          */
/* PAGE 4    get stream (copy)                   */
/*                                               */
/* ERROR 1   put stream pag1 did not validate    */
/* ERROR 2   put stream page 1 & 2 differ        */
/* ERROR 3   get stream values are out of range  */
/* --------------------------------------------- */

void ctrl_ramp_test()
{
 FILE *stream;
 FILE *logstream;
 unsigned put_stream[2048];
 unsigned get_stream[2048];
 unsigned adval;
 unsigned aimb1_low;
 unsigned aimb1_high;
 unsigned ambef_low;
 int error;
 int keep_looping;
 int loop_count;
 int range;
 char key;
 int v;
 int i;
 int j;
 char LSByte;
 char MSByte;

 init_timer();

 aimb1_low  = *(ISASEL_AON + AIMB1LOW);
 aimb1_high = *(ISASEL_AON + AIMB1HIGH);
		// flush the incoming mailbox

/* ---------------------------------------------------------- */
/*  set the DAC to zero volts, then read the A/D a few times  */
/* ---------------------------------------------------------- */


 for ( i=0; i < 5; i++) {
    *(ISASEL_PIO + 0x6) = 0;
    *(ISASEL_PIO + 0x7) = 0;

    while ( (*ISASEL_CPLD & 0x0001) == 0x00001 ) ;
       // loop till TMROUT0 = 0

    while ( (*ISASEL_CPLD & 0x0001) == 0x00000 ) ;
       // loop till TMROUT0 = 1

    get_stream[ i ] = *(ISASEL_PIO + 0x2);
 }


 /* open the log file */
 logstream = fopen("log.txt", "w+");

 keep_looping = 1;
 loop_count = 1;

/* stress test, add loop_count term to while statement */

 while( keep_looping && loop_count < 3000 ){
// while( 1 ){
 printf("%d wait for host ... ", loop_count);

 /* stress test */

 do {
    ambef_low = *(ISASEL_AON + AMBEFLOW) & 0x00f ;
    if ( loop_count > 5 ){

        do {
           *(ISASEL_TMR+3) = 0xd2;       /* Read Back Cmd, Latch cnt0 */
           LSByte =  *(ISASEL_TMR);      /* Cntr 0 */
           MSByte =  *(ISASEL_TMR);
           adval = *(ISASEL_PIO + 0x2);
           }
           while ( (*ISASEL_CPLD & 0x0001) == 0x00001 ) ;
              // loop till TMROUT0 = 0

         do {
           *(ISASEL_TMR+3) = 0xd2;       /* Read Back Cmd, Latch cnt0 */
           LSByte =  *(ISASEL_TMR);      /* Cntr 0 */
           MSByte =  *(ISASEL_TMR);
           adval = *(ISASEL_PIO + 0x2);
           }
           while ( (*ISASEL_CPLD & 0x0001) == 0x00000 ) ;
              // loop till TMROUT0 = 1

        } // if
    } // do ()
	while (  ambef_low != 0xf ) ;
		  /* loop till incoming mailbox IMB1 is full */

 printf("received commnad, process ambef_low = %x\n", ambef_low );


 aimb1_low  = *(ISASEL_AON + AIMB1LOW);
 aimb1_high = *(ISASEL_AON + AIMB1HIGH);
		// flush the incoming mailbox

 error = 0;    // initialize error code

 fprintf (logstream,"loop_count = %d\n", loop_count );

 /* read put_stream from shared memory */
 *ISASEL_PAGEREG = 1;

 for ( i = 0; i < 2048 ; i++ ){
      put_stream[ i ] = *( ISASEL_MEM + i );
      }


 /* ---------------------------------- */
 /*  verify that put_stream is correct */
 /* ---------------------------------- */


 /* move SAWTOOTH from zero to max positive value */

   v = 0;
	for ( i = 0; i < 512; i++ ){

      if ( put_stream [ i ] != v ){
         error = 1;       // error code
         fprintf (logstream,"put_stream [ %d ] = %d should be = %d\n",
             i, put_stream[ i ], v );
         put_stream[ i ] = *( ISASEL_MEM + i );
         }

      v = v+64;
		}

   /* move SAWTOOTH from max positive to max negative value */
   v = 32704;
	for ( i = 512; i < 1536; i++ ){

      if ( put_stream [ i ] != v ){
         error = 1;       // error code
         fprintf (logstream,"put_stream [ %d ] = %d should be = %d\n",
             i, put_stream[ i ], v );
         put_stream[ i ] = *( ISASEL_MEM + i );
         }

      v=v-64;
		}

   /* move SAWTOOTH from max negative to 1 count less than zero */
   v = 32768;
	for ( i = 1536; i < 2048; i++ ){

      if ( put_stream [ i ] != v ){
         error = 1;       // error code
         fprintf (logstream,"put_stream [ %d ] = %d should be = %d\n",
             i, put_stream [ i ], v );
         put_stream[ i ] = *( ISASEL_MEM + i );
         }

      v = v + 64;
		}

 /* open pstream.txt for update */
 stream = fopen("pstream.txt", "w+");
 fprintf (stream,"loop_count = %d\n", loop_count );

 for ( i = 0; i < 2048 ; i++ ){
      fprintf (stream,"%05u\n", put_stream[ i ] );
      }

 /* close the file */
 fclose(stream);

 /* open putval.txt for update */

 stream = fopen("putval.txt", "w+");

 for ( i = 0; i < 2048 ; i++ ){
      fprintf (stream,"%d\n", (int) put_stream[ i ] );
      }

 /* close the file */
 fclose(stream);

/* ----------------------------------- */
/* check that they are equal           */
/*   put stream page 1                 */
/*   put stream page 2                 */
/* ----------------------------------- */

 *ISASEL_PAGEREG = 2;

 for ( i = 0; i < 2048 ; i++ ){
      if ( put_stream[ i ] != *( ISASEL_MEM + i )){
           error = 2;
           fprintf(logstream,"put stream page 1 & 2 differ\n");
           }
      }

/* --------------------------------------------------- */
/*  write put_stream to DAC, read A/D into get_stream  */
/* --------------------------------------------------- */

 for ( i=0; i < 2048; i++) {
    *(ISASEL_PIO + 0x6) = put_stream[ i ];
    *(ISASEL_PIO + 0x7) = 0;

    while ( (*ISASEL_CPLD & 0x0001) == 0x00001 ) ;
       // loop till TMROUT0 = 0

    while ( (*ISASEL_CPLD & 0x0001) == 0x00000 ) ;
       // loop till TMROUT0 = 1

  get_stream[ i ] = *(ISASEL_PIO + 0x2);
 }


 /* check for out of range values in getstream */

 stream = fopen("range.txt", "w+");

 for ( j = 0; j < 2048; j++ ){
   range = (int) get_stream [ j ] - (int) put_stream [ j ];
   if ( (range > 1500) || (range < -1500) ){
      error = 3;
      fprintf (logstream,
           "out of range val, get_stream[%d] = %u error(%d)\n",
           j, get_stream[ j ], error );
      }
 fprintf (stream,"%d %d %d\n", (int) get_stream[j], range, error );
 }

 /* close the file */
 fclose(stream);

 /* write get_stream to shared memory, page 3 */

 *ISASEL_PAGEREG = 3;
 for ( i = 0; i < 2048 ; i++ ){
      *( ISASEL_MEM + i ) = get_stream[ i ] ;
      }

 /* open gstream.txt update */
 stream = fopen("gstream.txt", "w+");

 for ( j = 0; j < 2048; j++ ){
   fprintf (stream,"%05u\n",  get_stream [ j ]  );
 }

 /* close the file */
 fclose(stream);


 /* open getval.txt for update */

 stream = fopen("getval.txt", "w+");

 for ( j = 0; j < 2048; j++ ){
   fprintf (stream,"%d\n", (int) get_stream [ j ] );
 }

 /* close the file */
 fclose(stream);

  /* write get_stream to shared memory, page 4 */
 *ISASEL_PAGEREG = 4;
 for ( i = 0; i < 2048 ; i++ ){
      *( ISASEL_MEM + i ) = get_stream[ i ] ;
      }

 /* notify host */
 *(ISASEL_AON + AOMB1LOW) = 0;

  loop_count++;

// stress test, don't terminate on error
//  if (error)
//    keep_looping = 0;

 }  // while (keep_looping)

 /* close the log file */
 fclose(logstream);
}
