#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <time.h>

#include "amcc.h"

/* ISA address map */
#define ISASEL_MEM     ((unsigned*)0xd8000000L)      /* 2 bytes */
#define ISASEL_PAGEREG ((unsigned*)0xda000000L)
#define ISASEL_PIO     ((unsigned*)0xd9000000L)
#define ISASEL_AON     ((unsigned*)0xdc000000L)
#define ISASEL_TMR     ((char*)0xdb000000L)          /* 1 byte */

/* Add-On Operation Registers */
#define AIMB1LOW  0x00   /* Add-On Incoming Mailbox Register #1 */
#define AIMB1HIGH 0x01   /* Add-On Incoming Mailbox Register #1 */
#define AIMB2LOW  0x02   /* Add-On Incoming Mailbox Register #2 */
#define AIMB2HIGH 0x03   /* Add-On Incoming Mailbox Register #2 */
#define AIMB3LOW  0x04   /* Add-On Incoming Mailbox Register #3 */
#define AIMB3HIGH 0x05   /* Add-On Incoming Mailbox Register #3 */
#define AIMB4LOW  0x06   /* Add-On Incoming Mailbox Register #4 */
#define AIMB4HIGH 0x07   /* Add-On Incoming Mailbox Register #4 */
#define AOMB1LOW  0x08   /* Add-On Outgoing Mailbox Register #1 */
#define AOMB1HIGH 0x09   /* Add-On Outgoing Mailbox Register #1 */
#define AOMB2LOW  0x0a   /* Add-On Outgoing Mailbox Register #2 */
#define AOMB2HIGH 0x0b   /* Add-On Outgoing Mailbox Register #2 */
#define AOMB3LOW  0x0c   /* Add-On Outgoing Mailbox Register #3 */
#define AOMB3HIGH 0x0d   /* Add-On Outgoing Mailbox Register #3 */
#define AOMB4LOW  0x0e   /* Add-On Outgoing Mailbox Register #4 */
#define AOMB4HIGH 0x0f   /* Add-On Outgoing Mailbox Register #4 */

#define AMBEFLOW	0x1A   /* Add-On Mailbox  Empty/Full Status */
#define AMBEFHIGH	0x1B   /* Add-On Mailbox  Empty/Full Status */

void write_addon_outgoing_mailbox();
void validate_addon_incoming_mailbox();
void ctrl_data_integrity();
void ctrl_tf_test();
void ctrl_tfmb_test();
void ctrl_read_mailbox_loop();

/* --- keyboard interrupt routine --- */
#ifdef __cplusplus
    #define __CPPARGS ...
#else
    #define __CPPARGS
#endif

extern void interrupt get_out(__CPPARGS); /*    interrupt prototype */
extern void interrupt (*oldfunc)(__CPPARGS); /* interrupt function pointer */

extern int looping;


void write_addon_outgoing_mailbox()
{
 unsigned aomb1low_val;    /* 2 bytes */
 unsigned aomb1high_val;
 unsigned aomb2low_val;
 unsigned aomb2high_val;
 unsigned aomb3low_val;
 unsigned aomb3high_val;
 unsigned aomb4low_val;
 unsigned aomb4high_val;

 char key;

 printf("Write to AOMB1 .. AOMB4  ....");

 /* Generate random values for AOMB1 .. AOMB4 */
 srand(16);

 aomb1low_val    = (unsigned)(0xff & rand());
 aomb1low_val   += (unsigned)(0xff & rand()) << 8;
 aomb1high_val   = (unsigned)(0xff & rand());
 aomb1high_val  += (unsigned)(0xff & rand()) << 8;

 aomb2low_val    = (unsigned)(0xff & rand());
 aomb2low_val   += (unsigned)(0xff & rand()) << 8;
 aomb2high_val   = (unsigned)(0xff & rand());
 aomb2high_val  += (unsigned)(0xff & rand()) << 8;

 aomb3low_val    = (unsigned)(0xff & rand());
 aomb3low_val   += (unsigned)(0xff & rand()) << 8;
 aomb3high_val   = (unsigned)(0xff & rand());
 aomb3high_val  += (unsigned)(0xff & rand()) << 8;

 aomb4low_val    = (unsigned)(0xff & rand());
 aomb4low_val   += (unsigned)(0xff & rand()) << 8;
 aomb4high_val   = (unsigned)(0xff & rand());
 aomb4high_val  += (unsigned)(0xff & rand()) << 8;

 *(ISASEL_AON + AOMB4LOW)  =  aomb4low_val;
 *(ISASEL_AON + AOMB4HIGH) =  aomb4high_val;

 *(ISASEL_AON + AOMB3LOW)  =  aomb3low_val;
 *(ISASEL_AON + AOMB3HIGH) =  aomb3high_val;

 *(ISASEL_AON + AOMB2LOW)  =  aomb2low_val;
 *(ISASEL_AON + AOMB2HIGH) =  aomb2high_val;

 *(ISASEL_AON + AOMB1LOW)  =  aomb1low_val;
 *(ISASEL_AON + AOMB1HIGH) =  aomb1high_val;

 printf(" done \n");
 printf ("type ENTER key to continue\n");
 scanf ( "%c", &key );

}

/* ------------------------------------------------------------- */
/*  void validate_addon_incoming_mailbox()                       */
/*  Read AIMB1..AIMB4 and compare with the values                */
/*  the host computer wrote to OMB1..OMB4                        */
/*  OMB1..OMB4   Output Mailbox Registers ( written to by host ) */
/*  AIMB1..AIMB4 AddOn Input Mailbox Regs ( read by ctrl comp. ) */
/* ------------------------------------------------------------- */

void validate_addon_incoming_mailbox()
{
 unsigned long omb1_val;    /* 4 bytes */
 unsigned long omb2_val;
 unsigned long omb3_val;
 unsigned long omb4_val;

 unsigned long aimb1_val;    /* 4 bytes */
 unsigned long aimb2_val;
 unsigned long aimb3_val;
 unsigned long aimb4_val;

 unsigned ambef_low;

 char key;

 *(ISASEL_AON + AIMB1LOW);
 *(ISASEL_AON + AIMB1HIGH);
		// flush the incoming mailbox

 printf("Wait till PCI Computer Writes to OMB1 .. OMB4\n");

 do ( ambef_low = *(ISASEL_AON + AMBEFLOW) & 0x00f );
		while (  ambef_low != 0xf  ) ;
 			  /* loop till incoming mailbox IMB1 is full */


 /* Recreate the host values written to OMB1 .. OMB4 */
 srand(14);

 omb1_val =  (unsigned long)(0xff & rand());
 omb1_val += (unsigned long)(0xff & rand()) << 8;
 omb1_val += (unsigned long)(0xff & rand()) << 16;
 omb1_val += (unsigned long)(0xff & rand()) << 24;

 omb2_val =  (unsigned long)(0xff & rand());
 omb2_val += (unsigned long)(0xff & rand()) << 8;
 omb2_val += (unsigned long)(0xff & rand()) << 16;
 omb2_val += (unsigned long)(0xff & rand()) << 24;

 omb3_val =  (unsigned long)(0xff & rand());
 omb3_val += (unsigned long)(0xff & rand()) << 8;
 omb3_val += (unsigned long)(0xff & rand()) << 16;
 omb3_val += (unsigned long)(0xff & rand()) << 24;

 omb4_val =  (unsigned long)(0xff & rand());
 omb4_val += (unsigned long)(0xff & rand()) << 8;
 omb4_val += (unsigned long)(0xff & rand()) << 16;
 omb4_val += (unsigned long)(0xff & rand()) << 24;

 /* Read the values in the add on incoming mailbox registers */
 aimb1_val =   ((unsigned long)(0xffff & *(ISASEL_AON + AIMB1HIGH)) << 16)
             +  (unsigned long)(0xffff & *(ISASEL_AON + AIMB1LOW));


 aimb2_val =   ((unsigned long)(0xffff & *(ISASEL_AON + AIMB2HIGH)) << 16)
             +  (unsigned long)(0xffff & *(ISASEL_AON + AIMB2LOW));

 aimb3_val =   ((unsigned long)(0xffff & *(ISASEL_AON + AIMB3HIGH)) << 16)
             +  (unsigned long)(0xffff & *(ISASEL_AON + AIMB3LOW));

 aimb4_val =   ((unsigned long)(0xffff & *(ISASEL_AON + AIMB4HIGH)) << 16)
             +  (unsigned long)(0xffff & *(ISASEL_AON + AIMB4LOW));

 printf ("Verify that the values OMB1..OMB4 match AIMB1..AIMB4\n");
 printf (
 "  OMB1..OMB4 output mailbox registers written to by the host computer\n");

 printf (
 "  AIMB1..AIMB4 addon input mailbox registers read by the control computer\n");

 printf("\nOMB1 = 0x%08lx  AIMB1 = 0x%08lx   ", omb1_val, aimb1_val);
 if(omb1_val == aimb1_val)
   printf("values match\n");
 else
   printf("values do not match\n");

 printf("OMB2 = 0x%08lx  AIMB2 = 0x%08lx   ", omb2_val, aimb2_val);
 if(omb2_val == aimb2_val)
   printf("values match\n");
 else
   printf("values do not match\n");

 printf("OMB3 = 0x%08lx  AIMB3 = 0x%08lx   ", omb3_val, aimb3_val);
 if(omb3_val == aimb3_val)
   printf("values match\n");
 else
   printf("values do not match\n");

 printf("OMB4 = 0x%08lx  AIMB4 = 0x%08lx   ", omb4_val, aimb4_val);
 if(omb4_val == aimb4_val)
   printf("values match\n");
 else
   printf("values do not match\n");

 printf ("type ENTER key to continue\n");
 scanf ( "%c", &key );
}

void ctrl_data_integrity( )
{
 FILE *stream;
 unsigned aimb1_low;
 unsigned ambef_low;
 int fail;
 long total_fails;

 *(ISASEL_AON + AIMB1LOW);
 *(ISASEL_AON + AIMB1HIGH);
		// flush the incoming mailbox

 stream = fopen("log", "w+");

 puts("Press <Shift><PrtSc> to terminate");

 /* save the old interrupt */
 oldfunc  = _dos_getvect(5);

 /* install interrupt handler */
 _dos_setvect(5,get_out);

 looping = 1;
 total_fails = 0;

 while(looping){
      do ( ambef_low = *(ISASEL_AON + AMBEFLOW) & 0x00f );
		while (  (ambef_low != 0xf) && looping  ) ;
 			  /* loop till incoming mailbox IMB1 is full */

      aimb1_low = *(ISASEL_AON + AIMB1LOW);
      if ( fail = ctrl_validate_memory(aimb1_low) ){
   	 	fprintf (stream, "test fails, number of failures = %d\n", fail);
         printf ("test fails, number of failures = %d\n", fail);
         total_fails++;
         }
	   else {
	   	fprintf (stream,"test passes, errors(%d)\n", total_fails);
         printf ("test passes, errors(%d)\n", total_fails);
         }
      ctrl_fill_memory( aimb1_low);

		*(ISASEL_AON + AOMB1LOW) = aimb1_low;
      }    /* while looping */

  /* restore to original interrupt routine */
  _dos_setvect(5,oldfunc);

  fclose(stream);
}


void ctrl_data_integrity_mem( )
{
 FILE *stream;
 unsigned aimb1_low;
 unsigned ambef_low;
 int fail;
 long total_fails;
 char key;

// flush the incoming mailbox

   *ISASEL_PAGEREG = 0;
   *(ISASEL_MEM+2) = 0;

// stream = fopen("log", "w+");

 puts("Press <Shift><PrtSc> to terminate");

 /* save the old interrupt */
 oldfunc  = _dos_getvect(5);

 /* install interrupt handler */
 _dos_setvect(5,get_out);

 looping = 1;
 total_fails = 0;

 while(looping){

  *ISASEL_PAGEREG = 0;

      do ( ambef_low = *(ISASEL_MEM+2) & 0x001 );
  		while (  (ambef_low != 0x1) && looping  ) ;
 			  /* loop till incoming mailbox IMB1 is full */

      aimb1_low = *(ISASEL_MEM + 4);
      printf("seed 4 = %d\n", aimb1_low);

      *(ISASEL_MEM+2) = 0;  /* clear command mailbox */


      if ( fail = ctrl_validate_memory_mem(aimb1_low) ){
//   	 	fprintf (stream, "test fails, number of failures = %d\n", fail);
         printf ("test fails, number of failures = %d\n", fail);
         total_fails++;
         }
	   else {
//	   	fprintf (stream,"test passes, errors(%d)\n", total_fails);
         printf ("test passes, errors(%d)\n", total_fails);
         }

      ctrl_fill_memory_mem( aimb1_low);


   *ISASEL_PAGEREG = 0;
   *(ISASEL_MEM+4) = aimb1_low;


   *(ISASEL_MEM) = 1;

      }    /* while looping */

  /* restore to original interrupt routine */
  _dos_setvect(5,oldfunc);

//  fclose(stream);
}

void ctrl_tf_test()
{
 unsigned aimb1_low;
 unsigned ambef_low;
 unsigned read_u_val;       /* 2 bytes */
 unsigned write_u_val;

 int fail;
 long total_fails;
 char key;

// flush the incoming mailbox

  *ISASEL_PAGEREG = 0;
  *(ISASEL_MEM+2) = 0;

  printf("ctrl_tf_test\n");

 looping = 1;
 total_fails = 0;

 while(looping){

  *ISASEL_PAGEREG = 0;

      do ( ambef_low = *(ISASEL_MEM+2) & 0x001 );
  		while (  (ambef_low != 0x1) && looping  ) ;
 			  /* loop till incoming mailbox IMB1 is full */

      aimb1_low = *(ISASEL_MEM + 4);
      printf("seed = %d\n", aimb1_low);

      *(ISASEL_MEM+2) = 0;  /* clear command mailbox */

      /* check if value in memory is correct */

      srand ( aimb1_low );

      fail = 0;
      *ISASEL_PAGEREG = 1;

      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      read_u_val = *(ISASEL_MEM);

      if (  read_u_val !=  write_u_val )
	       fail = fail + 1;

      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      read_u_val = *(ISASEL_MEM+1);

      if (  read_u_val !=  write_u_val )
	       fail = fail + 1;


      if ( fail ){
         printf ("test fails, number of failures = %d\n", fail);
         total_fails++;
         }
	   else {
         printf ("test passes, errors(%d)\n", total_fails);
         }

      srand ( aimb1_low );
      *ISASEL_PAGEREG = 1;

      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      *(ISASEL_MEM) = write_u_val;

      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      *(ISASEL_MEM+1) = write_u_val;

   *ISASEL_PAGEREG = 0;
   *(ISASEL_MEM+4) = aimb1_low;


   *(ISASEL_MEM) = 1;

      }    /* while looping */

}

void ctrl_tfmb_test()
{
 unsigned aimb1_low;
 unsigned ambef_low;
 unsigned read_u_val;       /* 2 bytes */
 unsigned write_u_val;

 int fail;
 long total_fails;
 char key;

 *(ISASEL_AON + AIMB1LOW);
 *(ISASEL_AON + AIMB1HIGH);
		// flush the incoming mailbox


 printf("ctrl_tfmb_test\n");

 looping = 1;
 total_fails = 0;

 while(looping){

      do ( ambef_low = *(ISASEL_AON + AMBEFLOW) & 0x00f );
		while (  (ambef_low != 0xf) && looping  ) ;
 			  /* loop till incoming mailbox IMB1 is full */

      aimb1_low = *(ISASEL_AON + AIMB1LOW);
      printf("seed = %d\n", aimb1_low);

      /* check if value in memory is correct */

      srand ( aimb1_low );

      fail = 0;
      *ISASEL_PAGEREG = 1;

      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      read_u_val = *(ISASEL_MEM);

      if (  read_u_val !=  write_u_val ){
         printf(" %x %x \n", write_u_val, read_u_val);
         *(ISASEL_TMR+3) = 0x36;   /* counter 0 */
	       fail = fail + 1;
          }
      else
         printf(" %x %x \n", write_u_val, read_u_val);

      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      read_u_val = *(ISASEL_MEM+1);

      if (  read_u_val !=  write_u_val ){
         printf(" %x %x \n", write_u_val, read_u_val);
         *(ISASEL_TMR+3) = 0x36;   /* counter 0 */
	       fail = fail + 1;
          }
      else
         printf(" %x %x \n", write_u_val, read_u_val);


      if ( fail ){
         printf ("test fails, number of failures = %d\n", fail);
         total_fails++;
         }
	   else {
         printf ("test passes, errors(%d)\n", total_fails);
         }

      srand ( aimb1_low );
      *ISASEL_PAGEREG = 1;

      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      *(ISASEL_MEM) = write_u_val;

      write_u_val  = (unsigned)(0xff & rand());
      write_u_val += (unsigned)(0xff & rand()) << 8;
      *(ISASEL_MEM+1) = write_u_val;

  		*(ISASEL_AON + AOMB1LOW) = aimb1_low;
      }    /* while looping */

}


void  ctrl_read_mailbox_loop()
{
  unsigned ambef_low;

  while ( 1 )
    {
     ambef_low = *(ISASEL_AON + AMBEFLOW);
     printf ("loop\n");
    }

}
