#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <time.h>

#include "amcc.h"

extern dword base_address_0;   /* PCI_CS_BASE_ADDRESS_0 - amcc */
extern dword base_address_1;   /* PCI_CS_BASE_ADDRESS_1 - memory */
extern dword base_address_2;   /* PCI_CS_BASE_ADDRESS_2 - pio */
extern dword base_address_3;   /* PCI_CS_BASE_ADDRESS_3 - cpld */
extern dword base_address_4;   /* PCI_CS_BASE_ADDRESS_4 - tmr */

void initAMCC()
{
/* parameters for the function find_pci_device() */
word vendor_id;
word device_id;
word index;
byte bus;                 /* Bus number of device */
byte device_and_function; /* Device Number bits 7-3,
			     Function bits 2-0*/

int bios_present;         /* TRUE if PCI BIOS found */

word vendor_identification_register;
word device_identification_register;


/* ------------------------------- */
/* AMCC Default Vendor/Device ID's */
/* ------------------------------- */
/* vendor_id = 0x10E8; */
/* device_id = 0x5933; */

 vendor_id = 0x1234;
 device_id = 0x5678;


 index = 0;

/* ----------------------------- */
/* Determine if PCI BIOS present */
/* ----------------------------- */

if (pci_bios_present(NULL, NULL, NULL) == SUCCESSFUL){
       bios_present=TRUE;
       }
     else {
       printf ("PCI BIOS not present\n");
       bios_present=FALSE;
       }


/* ------------------------------------------------ */
/* If the BIOS is present, look for the AMCC device */
/* ------------------------------------------------ */

if(bios_present){
   if(find_pci_device( device_id, vendor_id, index, &bus,
	  &device_and_function)==SUCCESSFUL){
	  printf("AMCC Device found: Bus=%d Device=%d Function=%d\n",
	       bus,(device_and_function>>3),(device_and_function &0x7));
   }
   else
	  printf("Device not found!!!\n");
}  /* if(bios_present) */

/* --------------------------- */
/* Read Vendor Identification */
/* --------------------------- */
if( read_configuration_word( bus,device_and_function,
       PCI_CS_VENDOR_ID, &vendor_identification_register)==SUCCESSFUL)

       printf ("Vendor Identification = %x\n",
	      vendor_identification_register);
else

       printf ("Error While Reading PCI_CS_VENDOR_ID\n");

/* ---------------------------- */
/* Read Device Identification   */
/* ---------------------------- */
if( read_configuration_word( bus,device_and_function,
       PCI_CS_DEVICE_ID, &device_identification_register)==SUCCESSFUL)

       printf ("Device Identification = %x\n",
	      device_identification_register);

else
       printf ("Error While Reading PCI_CS_DEVICE_ID\n");



/* -------------------- */
/* Read Base Address 0  */
/* -------------------- */

if (read_configuration_dword(bus,device_and_function,
     PCI_CS_BASE_ADDRESS_0, &base_address_0)==SUCCESSFUL){
     /*Determine if I/O Address*/
     if(base_address_0 & 0x01)
     {
     /*Mask off bits 1-0*/
		base_address_0 &= 0xfffffffcL;
		printf("I/O Base Address Reg 0 = %lx\n", base_address_0);
     }
     else{
     /*Mask off bits 3-0*/
     base_address_0 = base_address_0 & 0xfffffff0L;
     printf("Memory Base Address Reg 0 = %lx\n", base_address_0);
     }
     }

/* ---------------------------------------- */
/*  Read Base Address 1, passthru region 0  */
/*  Memory                                  */
/* ---------------------------------------- */


if (read_configuration_dword(bus,device_and_function,
     PCI_CS_BASE_ADDRESS_1, &base_address_1)==SUCCESSFUL){
     /*Determine if I/O Address*/
     if(base_address_1 & 0x01) {
     /*Mask off bits 1-0*/
     base_address_1 &= 0xfffffffcL;
     printf("I/O Base Address Reg 1 (memory) = %lx\n", base_address_1);
     }
     else{
     /*Mask off bits 3-0*/
     base_address_1 = base_address_1 & 0xfffffff0L;
     printf("Memory Base Address Reg 1 (memory) = %lx\n", base_address_1);
     }
     }


/* --------------------------------------- */
/* Read Base Address 2, pass thru region 1 */
/*              PIO Precision Input/Output                       */
/* --------------------------------------- */

if (read_configuration_dword(bus,device_and_function,
     PCI_CS_BASE_ADDRESS_2, &base_address_2)==SUCCESSFUL){
     /*Determine if I/O Address*/
     if(base_address_2 & 0x01) {
     /*Mask off bits 1-0*/
     base_address_2 &= 0xfffffffcL;
     printf("I/O Base Address Reg 2 (PIO) = %lx\n", base_address_2);
     }
     else{
     /*Mask off bits 3-0*/
     base_address_2 = base_address_2 & 0xfffffff0L;
     printf("Memory Base Address Reg 2 (PIO) = %lx\n", base_address_2);
     }
     }

/* --------------------------------------- */
/* Read Base Address 3, pass thru region 2 */
/*      CPLD                                                                                     */
/* --------------------------------------- */

if (read_configuration_dword(bus,device_and_function,
     PCI_CS_BASE_ADDRESS_3, &base_address_3)==SUCCESSFUL){
     /*Determine if I/O Address*/
     if(base_address_3 & 0x01) {
     /*Mask off bits 1-0*/
     base_address_3 &= 0xfffffffcL;
     printf("I/O Base Address Reg 3 (CPLD) = %lx\n", base_address_3);
     }
     else{
     /*Mask off bits 3-0*/
     base_address_3 = base_address_3 & 0xfffffff0L;
     printf("Memory Base Address reg 3 (CPLD) = %lx\n", base_address_3);
     }
     }

/* --------------------------- */
/*  Read Base Address 4,       */
/*  pass thru region 3         */
/*  TMR                                                          */
/* --------------------------- */

if (read_configuration_dword(bus,device_and_function,
     PCI_CS_BASE_ADDRESS_4, &base_address_4)==SUCCESSFUL){
     /*Determine if I/O Address*/
     if(base_address_4 & 0x01) {
     /*Mask off bits 1-0*/
     base_address_4 &= 0xfffffffcL;
     printf("I/O Base Address Reg 4 (TMR) = %lx\n", base_address_4);
     }
     else{
     /*Mask off bits 3-0*/
     base_address_4 = base_address_4 & 0xfffffff0L;
     printf("Memory Base Address reg 4 (TMR) = %lx\n", base_address_4);
     }
     }

}
